@@ -312,14 +312,16 @@ sub pagetemplate (@) {
 		$template->param(title_overridden => 1);
 	}
 
-	foreach my $field (qw{author authorurl permalink}) {
+	foreach my $field (qw{author authorurl}) {
 		$template->param($field => $pagestate{$page}{meta}{$field})
 			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
 	}
 
 	foreach my $field (qw{permalink}) {
-		$template->param($field => IkiWiki::urlabs($pagestate{$page}{meta}{$field}, $config{url}))
-			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
+		if (exists $pagestate{$page}{meta}{$field} && $template->query(name => $field)) {
+			eval q{use HTML::Entities};
+			$template->param($field => HTML::Entities::encode_entities(IkiWiki::urlabs($pagestate{$page}{meta}{$field}, $config{url})));
+		}
 	}
 
 	foreach my $field (qw{description}) {
